/*
  LOGIC CHIP TESTER - for Touch Screen model
  V3: Uses standard Touch Screen Model I (blue base, sits forward close to row of digital pins)
      Maps Chip Pin #1 to Arduino Digital Pin #30. (Set with the FP constant within the code.)
      Each of the possible chips have a separate *.h file.
      There is no [Help] Button, but instead a D flip-flop test function

  Formats the buttons on the 3.5 inch TFT touch screen to display the logic chips that can be tested. 
  The chip tests are true/false function calls from the respective "*.h" header files. 

  Conditions:
  If a chip passes the functionality test, the screen will display GREEN

  Last Modified: May 27, 2021
  By Dr. Tim Frank
  with contributions from students Noel Rojas and Stephen Bakle

  This code is for the Glendale Community College Engineering Department.
*/

uint16_t  identifier    = 0x9486;   // Touchscreen driver chip                                                    
boolean   chipTest      = false;
int       JKTest        = 0;
const int FP            = 30;      // First Pin Number (Digital input pins on Arduino Board)
const int resultDelay   = 2000;
const int readDelay(100);

/**********************************************************************************************************
--------------------------------- IMPORT MAIN GRAPHICS AND TOUCHSCREEN LIBRARIES ---------------------------------
/**********************************************************************************************************/


#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_TFTLCD.h> // Hardware-specific library
#include <TouchScreen.h>
#include <MCUFRIEND_kbv.h>
          MCUFRIEND_kbv tft;
/**********************************************************************************************************
      --------------------------------- IMPORT TEST FILES ---------------------------------
/**********************************************************************************************************/
#include "AND.h"
#include "BUFFER.h"
#include "D_FlipFlop.h"
#include "JK_FlipFlop.h"
#include "NAND.h"
#include "NOR.h"
#include "NOT.h"
#include "OR.h"
#include "XOR.h"

/**********************************************************************************************************
      --------------------------------- Touchscreen Pins ---------------------------------
/**********************************************************************************************************/
#define LCD_CS              A3                         // Chip Select goes to Analog 3
#define LCD_CD              A2                         // Command/Data goes to Analog 2
#define LCD_WR              A1                         // LCD Write goes to Analog 1
#define LCD_RD              A0                         // LCD Read goes to Analog 0
#define LCD_RESET           A4                         // LCD Reset goes to Analog 4. Alternately be connected to Arduino's reset pin
/**********************************************************************************************************
      --------------------------------- COLOR DEFINITIONS (RGB 565) ---------------------------------
/**********************************************************************************************************/
#define BLACK               0x0000
#define WHITE               0xFFFF
#define BLOODRED            0x6861
#define RED                 0xF800
#define ORANGE              0xFC40
#define YELLOW              0xFF00
#define GREEN               0x07E0
#define GOLD                0xFE00
#define BLUE                0x001F
#define BUTTON              0xFF12
/**********************************************************************************************************
      --------------------------------- BUTTON SETTINGS ---------------------------------
/**********************************************************************************************************/
#define BUTTON_X            62                        //X starting position
#define BUTTON_Y            300                       //Y Starting Position
#define BUTTON_W            85                        //Button Width
#define BUTTON_H            45                        //Button Height
/*-------------------------------*/
#define BUTTON_SPACING_X    18
#define BUTTON_SPACING_Y    25
#define BUTTON_TEXTSIZE     2
/**********************************************************************************************************
      --------------------------------- TEXT-BOX SETTINGS ---------------------------------
/**********************************************************************************************************/
#define TEXT_X              10                        //X starting position
#define TEXT_Y              20                        //Y starting position
#define TEXT_W              300                       //Button Width
#define TEXT_H              230                       //Button Height
/*-------------------------------*/
#define TEXT_TSIZE          4        
#define TEXT_TCOLOR         WHITE
#define TEXT_LEN            6                       // Number of characters alHIGHed
/**********************************************************************************************************
      --------------------------------- TEXT STARTING POSITION ---------------------------------
/**********************************************************************************************************/
#define STATUS_X            60
#define STATUS_Y            65
/**********************************************************************************************************
      --------------------------------- ADAFRUIT - CREATE A BUTTON ARRAY---------------------------------
/**********************************************************************************************************/
Adafruit_GFX_Button buttons[9];
/**********************************************************************************************************
      --------------------------------- DEFINE BUTTON LABELS ---------------------------------
/**********************************************************************************************************/
char buttonlabels[9][7] = { 
                            "AND",      "OR",     "NOT", 
                            "NAND",     "NOR",    "XOR", 
                            "BUFFER",   "JK ff",  "D ff", 
                          };
/**********************************************************************************************************
      --------------------------------- DEFINE BUTTON COLORS ---------------------------------
/**********************************************************************************************************/                           
uint16_t buttoncolors[9] = {
                             BUTTON,    BUTTON,   BUTTON, 
                             BUTTON,    BUTTON,   BUTTON, 
                             BUTTON,    BUTTON,   BUTTON, 
                           };
/**********************************************************************************************************
      --------------------------------- TOUCH-SCREEN DEFINITIONS ---------------------------------
/**********************************************************************************************************/
/**
 * Note: The touch screen works by sending a digital signal through the X and Y axis, 
 * and then checking the resistance through the analog pin to obtain the position. 
 */
#define YP                  A2                      // Y-Plus - must be an analog pin, use "An" notation!
#define XM                  A3                      // X-Minus - must be an analog pin, use "An" notation!
#define YM                  8                       // Y-Minus - can be a digital pin
#define XP                  9                       // X-Plus - can be a digital pin
#define OHMS                300
/*-------------------------------*/
#define TS_MINX             75   
#define TS_MAXX             905
#define TS_MINY             75
#define TS_MAXY             930
/*-------------------------------*/
#define MINPRESSURE         100
#define MAXPRESSURE         1000                         
/**********************************************************************************************************
 ------------------------- MCUFRIEND - CREATE A TOUCH SCREEN OBJECT WITH THE PINS DEFINED -----------------
/**********************************************************************************************************/  
TouchScreen ts = TouchScreen(XP, YP, XM, YM, OHMS);  

/**********************************************************************************************************
 --------------------------------- MAIN LOOP, TOUCH SCREEN CONTROLS -----------------------------
/**********************************************************************************************************/  
void loop(void) 
{
      digitalWrite(13, HIGH);                     //Turn on Serial Clock
      TSPoint point = ts.getPoint();              //Create an Object called "Point" that gathers the X,Y,Z coordinates
      digitalWrite(13, HIGH);                      //Turn off Serial Clock
      pinMode(XM, OUTPUT);
      pinMode(YP, OUTPUT);

      /**********************************************************************************************************
         ------------------------ CHECK IF THE BUTTON WAS PRESSED WITH ENOUGH PRESSURE --------------------
      /**********************************************************************************************************/  
      if (point.z > MINPRESSURE && point.z < MAXPRESSURE) 
      {
            point.x = point.x + point.y;       
            point.y = point.x - point.y;            
            point.x = point.x - point.y; 
            // Original touch screen orientation  
//            point.x = map(point.x, TS_MINX, TS_MAXX, tft.width(), 0);
//            point.y = tft.height()-(map(point.y, TS_MINY, TS_MAXY, tft.height(), 0)); 
            //  Flip Touch points on Screen vertically.
            point.x = tft.width()-map(point.x, TS_MINX, TS_MAXX, tft.width(), 0);
            point.y = map(point.y, TS_MINY, TS_MAXY, tft.height(), 0); 
      }

      /**********************************************************************************************************
       --------------------- CHECK IF THE BUTTON WAS PRESSED WITHIN THE BOUNDS OF THE BUTTON --------------------
      /**********************************************************************************************************/  
      for (uint8_t b=0; b<9; b++) 
      {
            if ((buttons[b].contains(point.x, point.y)) && point.x>10) 
            {
                  Serial.print("Pressing: "); Serial.println(b);
                  buttons[b].press(true);  // tell the button it is pressed
            } 
            else 
            {
                  buttons[b].press(false);  // tell the button it is NOT pressed
            }
      }
      /**********************************************************************************************************
                --------------------- CHECK IF THE BUTTON STATE HAS CHANGED --------------------
      /**********************************************************************************************************/  
      for (uint8_t b=0; b<9; b++) 
      {
                  /**********************************************************************************************************
                   --------------------- IF THE BUTTON WAS RELEASED, DISPLAY "RELEASED" ON SERIAL MONITOR -------------------
                  /**********************************************************************************************************/  
                  if (buttons[b].justReleased()) 
                  {
                     Serial.print("Released: "); Serial.println(b);
                     buttons[b].drawButton();  // draw normal
                  }
                  /**********************************************************************************************************
                   --------------------- CHECK IF THE BUTTON WAS PRESSED WITHIN THE BUTTON REGION --------------------
                  /**********************************************************************************************************/            
                  if (buttons[b].justPressed()) 
                  {
                      buttons[b].drawButton(true);                          // Invert the color to signify that the button was pressed
                      tft.setCursor(TEXT_X + 2, TEXT_Y+10);                 // Update the cursor starting position
                      tft.setTextColor(TEXT_TCOLOR, BLACK);                 // Set the color
                      tft.setTextSize(TEXT_TSIZE);                          // Set the text size
                      
                      switch(b)                                             // Check which test was selected
                      {
                            /**********************************************************************************************************
                                                    --------------------- AND GATE DISPLAY --------------------
                            /**********************************************************************************************************/
                            case 0: 
                            {
                                status(F("AND GATE\n     7408"));
                                tft.setCursor(STATUS_X+45, STATUS_Y);
                                delay(500);
                                Serial.println("\n*** AND chip ***\n");
                                chipTest = AND();
                                PFdisplay(chipTest);
                                break;
                            }   
                            /**********************************************************************************************************
                                                    --------------------- OR GATE DISPLAY --------------------
                            /**********************************************************************************************************/
                            case 1:
                            {
                                status(F(" OR GATE\n     7432"));
                                tft.setCursor(STATUS_X+45, STATUS_Y);
                                delay(500);
                                Serial.println("\n*** OR chip ***\n");
                                chipTest = OR();
                                PFdisplay(chipTest);
                                break;
                            }
                            /**********************************************************************************************************
                                                    --------------------- NOT GATE DISPLAY --------------------
                            /**********************************************************************************************************/
                            case 2: 
                            {
                                status(F("NOT GATE\n     7404"));
                                tft.setCursor(STATUS_X+45, STATUS_Y);
                                delay(500);
                                Serial.println("\n*** NOT chip ***\n");
                                chipTest = NOT();
                                PFdisplay(chipTest);
                                break;
                            }
                            /**********************************************************************************************************
                                                    --------------------- NAND GATE DISPLAY --------------------
                            /**********************************************************************************************************/
                            case 3:  
                            {
                                status(F("NAND GATE\n     7400"));
                                tft.setCursor(STATUS_X+45, STATUS_Y);
                                delay(500);
                                Serial.println("\n*** NAND chip ***\n");
                                chipTest = NAND();
                                PFdisplay(chipTest);
                                break;
                            }
                            /**********************************************************************************************************
                                                    --------------------- NOR GATE DISPLAY --------------------
                            /**********************************************************************************************************/
                            case 4:  
                            {
                                status(F("NOR GATE\n     7402"));
                                tft.setCursor(STATUS_X+45, STATUS_Y);
                                delay(500);
                                Serial.println("\n*** NOR chip ***\n");
                                chipTest = NOR();
                                PFdisplay(chipTest);
                                break;
                            }
                            /**********************************************************************************************************
                                                    --------------------- XOR GATE DISPLAY --------------------
                            /**********************************************************************************************************/
                            case 5:  
                            {
                                status(F("XOR GATE\n     7486"));
                                tft.setCursor(STATUS_X+45, STATUS_Y);
                                delay(500);
                                Serial.println("\n*** XOR chip ***\n");
                                chipTest = XOR();
                                PFdisplay(chipTest);
                                break;
                            }
                            /**********************************************************************************************************
                                                    --------------------- BUFFER GATE DISPLAY --------------------
                            /**********************************************************************************************************/
                            case 6:  
                            {
                                status(F(" BUFFER\n    74126"));
                                tft.setCursor(STATUS_X+45, STATUS_Y);
                                delay(500);
                                Serial.println("\n*** BUFFER chip ***\n");
                                chipTest = BUFFER();
                                PFdisplay(chipTest);
                                break;
                            }
                            /**********************************************************************************************************
                                                    --------------------- JK FLIP-FLOP DISPLAY --------------------
                            /**********************************************************************************************************/
                    
                            case 7:  
                            {
                                status(F("JK F-Flop\n     7476"));
                                tft.setCursor(STATUS_X+45, STATUS_Y);
                                delay(500);
                                Serial.println("\n*** JK chip ***\n");
                                JKTest = JK_FlipFlop();
                                if(JKTest == 2) // JK passed but has Synchronization Issues
                                {
                                    tft.fillRect(TEXT_X+5, TEXT_Y+5, TEXT_W-(5*2), TEXT_H-(5*2), YELLOW);
                                    tft.setCursor(STATUS_X-33, STATUS_Y+10);
                                    tft.setTextSize(3);
                                    tft.print("Synchronization");
                                    tft.setCursor(STATUS_X+50, STATUS_Y+45);
                                    tft.print("Issues");
                                    readDelay;
                                    pinLow(); // Set all pins back to 0V
                                    delay(resultDelay); 
                                    screen();
                                }
                                else  // JK chip either failed or passed (with no synchronization issues).
                                {
                                    chipTest = (boolean)JKTest;
                                    PFdisplay(chipTest);
                                }
                                break;
                            }
                            /**********************************************************************************************************
                                                    --------------------- HELP DISPLAY --------------------
                            /**********************************************************************************************************/
                            case 8:  
                            {
                                status(F("D F-Flop\n     7474"));
                                tft.setCursor(STATUS_X+45, STATUS_Y);
                                delay(500);
                                Serial.println("\n*** D chip ***\n");
                                chipTest = D_FlipFlop();
                                PFdisplay(chipTest);
                                break;
/*
                              //status(F("HELP"));
                              tft.fillRect(TEXT_X+5, TEXT_Y+5, TEXT_W-(5*2), TEXT_H-(5*2), WHITE);
                              tft.setTextColor(BLACK);
                              tft.setTextSize(3);
                              tft.print("  Instructions:\n");
                              tft.setTextSize(1);
                              tft.print("\n");
                              tft.setTextSize(2);
                              tft.print("  1) Set the Chip in the\n");
                              tft.print("   socket with the notch\n");
                              tft.print("   next to the lever. \n");
                              tft.print("   Push the lever down \n");
                              tft.print("   to lock in place.\n");
                              tft.setTextSize(1);
                              tft.print("\n");
                              tft.setTextSize(2);
                              tft.print("  2) Select the button \n");
                              tft.print("   for the type of Chip.\n");
                              tft.setTextSize(1);
                              tft.print("\n");
                              tft.setTextSize(2);
                              tft.print("  3) Raise the lever to \n");
                              tft.print("   remove the Chip when\n");
                              tft.print("   testing is finished.");
                      //  *** To test connectivity. Comment out call to D_FlipFlop() above (within Case 8) and then 
                      //  *** uncomment the Help menu and the line below. Pressing Help will output 5V on all pins.
                              pinHighTest(); // Tests pin connectivity. All pins output High.
                              break;
*/
                            }
                          delay(100); // UI debouncing
              }
        }
      }
}

/**********************************************************************************************************
                --------------------- STATUS MESSAGE DISPLAY --------------------
/**********************************************************************************************************/  
void status(const __FlashStringHelper *msg) 
{
  tft.fillRect(TEXT_X+5, TEXT_Y+5, TEXT_W-(5*2), TEXT_H-(5*2), WHITE);
  tft.setCursor(STATUS_X, STATUS_Y);
  tft.setTextColor(BLACK);
  tft.setTextSize(4);
  tft.print(msg);
}

/**********************************************************************************************************
                --------------------- PASS/FAIL MESSAGE DISPLAY --------------------
/**********************************************************************************************************/  
void PFdisplay(boolean chipResult) 
{
  if(chipResult == true)
  {
        tft.fillRect(TEXT_X+5, TEXT_Y+5, TEXT_W-(5*2), TEXT_H-(5*2), GREEN);
        tft.setTextSize(5);
        tft.print("PASS");
        delay(3000);
  }
  else
  {
        tft.fillRect(TEXT_X+5, TEXT_Y+5, TEXT_W-(5*2), TEXT_H-(5*2), RED);
        tft.setTextSize(5);
        tft.setTextColor(WHITE);
        tft.print("FAIL");
        readDelay;
  }
  pinLow(); // Set all pins back to 0V
  delay(resultDelay); 
  screen();
}

/**********************************************************************************************************
                --------------------- INITIAL SCREEN MESSAGE --------------------
/**********************************************************************************************************/  
void screen(void) 
{
  tft.fillRect(TEXT_X+5, TEXT_Y+5, TEXT_W-(5*2), TEXT_H-(5*2), WHITE);
  tft.setCursor(STATUS_X+45, STATUS_Y);
  tft.setTextColor(BLUE);
  tft.setTextSize(1);
  tft.print("\n");
  tft.setTextSize(4);
  tft.print("  Logic Chip\n");
  tft.setTextSize(1);
  tft.print("\n");
  tft.setTextSize(4);
  tft.print("    Tester\n\n");
  tft.setTextColor(RED);
  tft.setTextSize(2);
  tft.print("  Insert and Select Type\n");
}

/**********************************************************************************************************
                --------------------- GENERATE BUTTONS --------------------
/**********************************************************************************************************/  
void setup(void) {
  Serial.begin(9600);                                // Read at the rate of 9600 bits per second
  
  tft.reset();                                       // Clear contents of the screen  
  tft.begin(identifier);
  tft.setRotation(2);   //Flip Screen  display vertically.
  tft.fillScreen(BLUE);
  tft.fillRect(TEXT_X, TEXT_Y, TEXT_W, TEXT_H, GOLD);
  screen();  // Display screen message
  
  for (uint8_t row=0; row<3; row++) 
  {
        for (uint8_t col=0; col<3; col++) 
        {
              buttons[col + row*3].initButton(      &tft, 
                                                    BUTTON_X+col*(BUTTON_W+BUTTON_SPACING_X), 
                                                    BUTTON_Y+row*(BUTTON_H+BUTTON_SPACING_Y), 
                                                    BUTTON_W, 
                                                    BUTTON_H, 
                                                    GOLD, 
                                                    buttoncolors[col+row*3], 
                                                    BLACK, 
                                                    buttonlabels[col + row*3], 
                                                    BUTTON_TEXTSIZE); 
              buttons[col + row*3].drawButton();
        }
  }
  pinLow(); // Set all pins initially to 0V
}

/**********************************************************************************************************
                --------------------- RESETS ALL PINS TO GND --------------------
/**********************************************************************************************************/  
void pinLow(void) { 
// This function resets all of pins to GND, (0V). 
  
                  /*   ------------   */  
  int P00=FP;     /*   |   \_/    |   */  int P01=FP+1;
  int P02=FP+2;   /*   |          |   */  int P03=FP+3;    
  int P04=FP+4;   /*   |          |   */  int P05=FP+5;
  int P06=FP+6;   /*   |          |   */  int P07=FP+7; 
  int P08=FP+8;   /*   |          |   */  int P09=FP+9;
  int P10=FP+10;  /*   |          |   */  int P11=FP+11; 
  int P12=FP+12;  /*   |          |   */  int P13=FP+13; 
  int P14=FP+14;  /*   |          |   */  int P15=FP+15;   
                  /*   ------------   */ 

  // Define all pins as Outputs
  pinMode(P00, OUTPUT); pinMode(P01, OUTPUT);
  pinMode(P02, OUTPUT); pinMode(P03, OUTPUT);
  pinMode(P04, OUTPUT); pinMode(P05, OUTPUT);
  pinMode(P06, OUTPUT); pinMode(P07, OUTPUT);
  pinMode(P08, OUTPUT); pinMode(P09, OUTPUT);
  pinMode(P10, OUTPUT); pinMode(P11, OUTPUT);
  pinMode(P12, OUTPUT); pinMode(P13, OUTPUT);
  pinMode(P14, OUTPUT); pinMode(P15, OUTPUT);
                                                                                              
  // Set all Pins to Initially Low
  digitalWrite(P00, LOW ); digitalWrite(P01, LOW );
  digitalWrite(P02, LOW ); digitalWrite(P03, LOW );
  digitalWrite(P04, LOW ); digitalWrite(P05, LOW );
  digitalWrite(P06, LOW ); digitalWrite(P07, LOW );
  digitalWrite(P08, LOW ); digitalWrite(P09, LOW );
  digitalWrite(P10, LOW ); digitalWrite(P11, LOW );
  digitalWrite(P12, LOW ); digitalWrite(P13, LOW );
  digitalWrite(P14, LOW ); digitalWrite(P15, LOW );
}

/**********************************************************************************************************
                --------------------- TEST CONEECTIVITY OF PINS --------------------
/**********************************************************************************************************/  
void pinHighTest(void) { 
// This function tests connectivity between Arduino and ZIF socket by outputting high voltages 
// on all pins until another button is pushed.
  
                  /*   ------------   */  
  int P00=FP;     /*   |   \_/    |   */  int P01=FP+1;
  int P02=FP+2;   /*   |          |   */  int P03=FP+3;    
  int P04=FP+4;   /*   |          |   */  int P05=FP+5;
  int P06=FP+6;   /*   |          |   */  int P07=FP+7; 
  int P08=FP+8;   /*   |          |   */  int P09=FP+9;
  int P10=FP+10;  /*   |          |   */  int P11=FP+11; 
  int P12=FP+12;  /*   |          |   */  int P13=FP+13; 
  int P14=FP+14;  /*   |          |   */  int P15=FP+15;   
                  /*   ------------   */ 

  // Define all pins as Outputs
  pinMode(P00, OUTPUT); pinMode(P01, OUTPUT);
  pinMode(P02, OUTPUT); pinMode(P03, OUTPUT);
  pinMode(P04, OUTPUT); pinMode(P05, OUTPUT);
  pinMode(P06, OUTPUT); pinMode(P07, OUTPUT);
  pinMode(P08, OUTPUT); pinMode(P09, OUTPUT);
  pinMode(P10, OUTPUT); pinMode(P11, OUTPUT);
  pinMode(P12, OUTPUT); pinMode(P13, OUTPUT);
  pinMode(P14, OUTPUT); pinMode(P15, OUTPUT);
                                                                                              
  // Set all Pins to high
  Serial.println("*** OUTPUTTING High Voltages on all pins ***");
  digitalWrite(P00, HIGH ); digitalWrite(P01, HIGH );
  digitalWrite(P02, HIGH ); digitalWrite(P03, HIGH );
  digitalWrite(P04, HIGH ); digitalWrite(P05, HIGH );
  digitalWrite(P06, HIGH ); digitalWrite(P07, HIGH );
  digitalWrite(P08, HIGH ); digitalWrite(P09, HIGH );
  digitalWrite(P10, HIGH ); digitalWrite(P11, HIGH );
  digitalWrite(P12, HIGH ); digitalWrite(P13, HIGH );
  digitalWrite(P14, HIGH ); digitalWrite(P15, HIGH );
  delay (2000);

  // Check for button being pushed to test a logic gate
  while(1){
    TSPoint point = ts.getPoint();      //Create an Object called "Point" that gathers the X,Y,Z coordinates of touched point
    if (point.z > MINPRESSURE && point.z < MAXPRESSURE) {
      pinLow(); // Set all pins back to 0V
      break; // end loop and return to main program
    }
  }
}
